@@ -88,12 +88,15 @@ namespace gbe {
 
   Kernel::Kernel(const std::string &name) :
     name(name), args(NULL), argNum(0), curbeSize(0), stackSize(0), useSLM(false),
-        slmSize(0), ctx(NULL), samplerSet(NULL), imageSet(NULL), printfSet(NULL) {}
+        slmSize(0), ctx(NULL), samplerSet(NULL), imageSet(NULL), printfSet(NULL),
+        profilingInfo(NULL), useDeviceEnqueue(false) {}
+
   Kernel::~Kernel(void) {
     if(ctx) GBE_DELETE(ctx);
     if(samplerSet) GBE_DELETE(samplerSet);
     if(imageSet) GBE_DELETE(imageSet);
     if(printfSet) GBE_DELETE(printfSet);
+    if(profilingInfo) GBE_DELETE(profilingInfo);
     GBE_SAFE_DELETE_ARRAY(args);
   }
   int32_t Kernel::getCurbeOffset(gbe_curbe_type type, uint32_t subType) const {
@@ -104,24 +107,46 @@ namespace gbe {
     return it->offset; // we found it!
   }
 
-  Program::Program(void) : constantSet(NULL) {}
+  Program::Program(uint32_t fast_relaxed_math) : fast_relaxed_math(fast_relaxed_math), 
+                               constantSet(NULL),
+                               relocTable(NULL) {}
   Program::~Program(void) {
     for (map<std::string, Kernel*>::iterator it = kernels.begin(); it != kernels.end(); ++it)
       GBE_DELETE(it->second);
     if (constantSet) delete constantSet;
+    if (relocTable) delete relocTable;
   }
 
 #ifdef GBE_COMPILER_AVAILABLE
   BVAR(OCL_OUTPUT_GEN_IR, false);
   BVAR(OCL_STRICT_CONFORMANCE, true);
+  IVAR(OCL_PROFILING_LOG, 0, 0, 1); // Int for different profiling types.
+  BVAR(OCL_OUTPUT_BUILD_LOG, false);
 
-  bool Program::buildFromLLVMFile(const char *fileName, const void* module, std::string &error, int optLevel) {
+  bool Program::buildFromLLVMFile(const char *fileName,
+                                         const void* module,
+                                         std::string &error,
+                                         int optLevel) {
     ir::Unit *unit = new ir::Unit();
     llvm::Module * cloned_module = NULL;
+    bool ret = false;
     if(module){
+#if LLVM_VERSION_MAJOR == 3 && LLVM_VERSION_MINOR >= 8
+      cloned_module = llvm::CloneModule((llvm::Module*)module).release();
+#else
       cloned_module = llvm::CloneModule((llvm::Module*)module);
+#endif
     }
-    if (llvmToGen(*unit, fileName, module, optLevel, OCL_STRICT_CONFORMANCE) == false) {
+    bool strictMath = true;
+    if (fast_relaxed_math || !OCL_STRICT_CONFORMANCE)
+      strictMath = false;
+#if LLVM_VERSION_MAJOR == 3 && LLVM_VERSION_MINOR >= 9
+    llvm::Module * linked_module = module ? llvm::CloneModule((llvm::Module*)module).release() : NULL;
+    // Src now will be removed automatically. So clone it.
+    if (llvmToGen(*unit, fileName, linked_module, optLevel, strictMath, OCL_PROFILING_LOG, error) == false) {
+#else
+    if (llvmToGen(*unit, fileName, module, optLevel, strictMath, OCL_PROFILING_LOG, error) == false) {
+#endif
       if (fileName)
         error = std::string(fileName) + " not found";
       delete unit;
@@ -134,31 +159,51 @@ namespace gbe {
       unit = new ir::Unit();
       if(cloned_module){
         //suppose file exists and llvmToGen will not return false.
-        llvmToGen(*unit, fileName, cloned_module, 0, OCL_STRICT_CONFORMANCE);
+        llvmToGen(*unit, fileName, cloned_module, 0, strictMath, OCL_PROFILING_LOG, error);
       }else{
         //suppose file exists and llvmToGen will not return false.
-        llvmToGen(*unit, fileName, module, 0, OCL_STRICT_CONFORMANCE);
+        llvmToGen(*unit, fileName, module, 0, strictMath, OCL_PROFILING_LOG, error);
       }
     }
-    assert(unit->getValid());
-    this->buildFromUnit(*unit, error);
+    if(unit->getValid()){
+      std::string error2;
+      if (this->buildFromUnit(*unit, error2)){
+        ret = true;
+      }
+      error = error + error2;
+    }
     delete unit;
     if(cloned_module){
       delete (llvm::Module*) cloned_module;
     }
-    return true;
+    return ret;
   }
 
   bool Program::buildFromUnit(const ir::Unit &unit, std::string &error) {
     constantSet = new ir::ConstantSet(unit.getConstantSet());
+    relocTable = new ir::RelocTable(unit.getRelocTable());
+    blockFuncs = unit.blockFuncs;
     const auto &set = unit.getFunctionSet();
     const uint32_t kernelNum = set.size();
     if (OCL_OUTPUT_GEN_IR) std::cout << unit;
     if (kernelNum == 0) return true;
+
+    bool strictMath = true;
+    if (fast_relaxed_math || !OCL_STRICT_CONFORMANCE)
+      strictMath = false;
+
     for (const auto &pair : set) {
       const std::string &name = pair.first;
-      Kernel *kernel = this->compileKernel(unit, name, !OCL_STRICT_CONFORMANCE);
+      Kernel *kernel = this->compileKernel(unit, name, !strictMath, OCL_PROFILING_LOG);
+      if (!kernel) {
+        error +=  name;
+        error += ":(GBE): error: failed in Gen backend.\n";
+        if (OCL_OUTPUT_BUILD_LOG)
+          llvm::errs() << error;
+        return false;
+      }
       kernel->setSamplerSet(pair.second->getSamplerSet());
+      kernel->setProfilingInfo(new ir::ProfilingInfo(*unit.getProfilingInfo()));
       kernel->setImageSet(pair.second->getImageSet());
       kernel->setPrintfSet(pair.second->getPrintfSet());
       kernel->setCompileWorkGroupSize(pair.second->getCompileWorkGroupSize());
@@ -172,17 +217,18 @@ namespace gbe {
 #define OUT_UPDATE_SZ(elt) SERIALIZE_OUT(elt, outs, ret_size)
 #define IN_UPDATE_SZ(elt) DESERIALIZE_IN(elt, ins, total_size)
 
-  size_t Program::serializeToBin(std::ostream& outs) {
-    size_t ret_size = 0;
-    size_t ker_num = kernels.size();
-    int has_constset = 0;
+  uint32_t Program::serializeToBin(std::ostream& outs) {
+    uint32_t ret_size = 0;
+    uint32_t ker_num = kernels.size();
+    uint32_t has_constset = 0;
+    uint32_t has_relocTable = 0;
 
     OUT_UPDATE_SZ(magic_begin);
 
     if (constantSet) {
       has_constset = 1;
       OUT_UPDATE_SZ(has_constset);
-      size_t sz = constantSet->serializeToBin(outs);
+      uint32_t sz = constantSet->serializeToBin(outs);
       if (!sz)
         return 0;
 
@@ -191,9 +237,21 @@ namespace gbe {
       OUT_UPDATE_SZ(has_constset);
     }
 
+    if(relocTable) {
+      has_relocTable = 1;
+      OUT_UPDATE_SZ(has_relocTable);
+      uint32_t sz = relocTable->serializeToBin(outs);
+      if (!sz)
+        return 0;
+
+      ret_size += sz;
+    } else {
+      OUT_UPDATE_SZ(has_relocTable);
+    }
+
     OUT_UPDATE_SZ(ker_num);
     for (map<std::string, Kernel*>::iterator it = kernels.begin(); it != kernels.end(); ++it) {
-      size_t sz = it->second->serializeToBin(outs);
+      uint32_t sz = it->second->serializeToBin(outs);
       if (!sz)
         return 0;
 
@@ -206,11 +264,12 @@ namespace gbe {
     return ret_size;
   }
 
-  size_t Program::deserializeFromBin(std::istream& ins) {
-    size_t total_size = 0;
+  uint32_t Program::deserializeFromBin(std::istream& ins) {
+    uint32_t total_size = 0;
     int has_constset = 0;
-    size_t ker_num;
+    uint32_t ker_num;
     uint32_t magic;
+    uint32_t has_relocTable = 0;
 
     IN_UPDATE_SZ(magic);
     if (magic != magic_begin)
@@ -219,19 +278,29 @@ namespace gbe {
     IN_UPDATE_SZ(has_constset);
     if(has_constset) {
       constantSet = new ir::ConstantSet;
-      size_t sz = constantSet->deserializeFromBin(ins);
+      uint32_t sz = constantSet->deserializeFromBin(ins);
 
-      if (sz == 0) {
+      if (sz == 0)
+        return 0;
+
+      total_size += sz;
+    }
+
+    IN_UPDATE_SZ(has_relocTable);
+    if(has_relocTable) {
+      relocTable = new ir::RelocTable;
+      uint32_t sz = relocTable->deserializeFromBin(ins);
+
+      if (sz == 0)
         return 0;
-      }
 
       total_size += sz;
     }
 
     IN_UPDATE_SZ(ker_num);
 
-    for (size_t i = 0; i < ker_num; i++) {
-      size_t ker_serial_sz;
+    for (uint32_t i = 0; i < ker_num; i++) {
+      uint32_t ker_serial_sz;
       std::string ker_name; // Just a empty name here.
       Kernel* ker = allocateKernel(ker_name);
 
@@ -246,7 +315,7 @@ namespace gbe {
     if (magic != magic_end)
       return 0;
 
-    size_t total_bytes;
+    uint32_t total_bytes;
     IN_UPDATE_SZ(total_bytes);
     if (total_bytes + sizeof(total_size) != total_size)
       return 0;
@@ -254,18 +323,22 @@ namespace gbe {
     return total_size;
   }
 
-  size_t Kernel::serializeToBin(std::ostream& outs) {
+  uint32_t Kernel::serializeToBin(std::ostream& outs) {
     unsigned int i;
-    size_t ret_size = 0;
+    uint32_t ret_size = 0;
     int has_samplerset = 0;
     int has_imageset = 0;
+    uint32_t sz = 0;
 
     OUT_UPDATE_SZ(magic_begin);
 
-    OUT_UPDATE_SZ(name.size());
+    sz = name.size();
+    OUT_UPDATE_SZ(sz);
     outs.write(name.c_str(), name.size());
     ret_size += sizeof(char)*name.size();
 
+    OUT_UPDATE_SZ(oclVersion);
+
     OUT_UPDATE_SZ(argNum);
     for (i = 0; i < argNum; i++) {
       KernelArgument& arg = args[i];
@@ -276,25 +349,30 @@ namespace gbe {
 
       OUT_UPDATE_SZ(arg.info.addrSpace);
 
-      OUT_UPDATE_SZ(arg.info.typeName.size());
+      sz = arg.info.typeName.size();
+      OUT_UPDATE_SZ(sz);
       outs.write(arg.info.typeName.c_str(), arg.info.typeName.size());
       ret_size += sizeof(char)*arg.info.typeName.size();
 
-      OUT_UPDATE_SZ(arg.info.accessQual.size());
+      sz = arg.info.accessQual.size();
+      OUT_UPDATE_SZ(sz);
       outs.write(arg.info.accessQual.c_str(), arg.info.accessQual.size());
       ret_size += sizeof(char)*arg.info.accessQual.size();
 
-      OUT_UPDATE_SZ(arg.info.typeQual.size());
+      sz = arg.info.typeQual.size();
+      OUT_UPDATE_SZ(sz);
       outs.write(arg.info.typeQual.c_str(), arg.info.typeQual.size());
       ret_size += sizeof(char)*arg.info.typeQual.size();
 
-      OUT_UPDATE_SZ(arg.info.argName.size());
+      sz = arg.info.argName.size();
+      OUT_UPDATE_SZ(sz);
       outs.write(arg.info.argName.c_str(), arg.info.argName.size());
       ret_size += sizeof(char)*arg.info.argName.size();
     }
 
-    OUT_UPDATE_SZ(patches.size());
-    for (size_t i = 0; i < patches.size(); ++i) {
+    sz = patches.size();
+    OUT_UPDATE_SZ(sz);
+    for (uint32_t i = 0; i < patches.size(); ++i) {
       const PatchInfo& patch = patches[i];
       unsigned int tmp;
       tmp = patch.type;
@@ -318,7 +396,7 @@ namespace gbe {
     if (!samplerSet->empty()) {   //samplerSet is always valid, allocated in Function::Function
       has_samplerset = 1;
       OUT_UPDATE_SZ(has_samplerset);
-      size_t sz = samplerSet->serializeToBin(outs);
+      uint32_t sz = samplerSet->serializeToBin(outs);
       if (!sz)
         return 0;
 
@@ -331,7 +409,7 @@ namespace gbe {
     if (!imageSet->empty()) {   //imageSet is always valid, allocated in Function::Function
       has_imageset = 1;
       OUT_UPDATE_SZ(has_imageset);
-      size_t sz = imageSet->serializeToBin(outs);
+      uint32_t sz = imageSet->serializeToBin(outs);
       if (!sz)
         return 0;
 
@@ -352,19 +430,19 @@ namespace gbe {
     return ret_size;
   }
 
-  size_t Kernel::deserializeFromBin(std::istream& ins) {
-    size_t total_size = 0;
+  uint32_t Kernel::deserializeFromBin(std::istream& ins) {
+    uint32_t total_size = 0;
     int has_samplerset = 0;
     int has_imageset = 0;
-    size_t code_size = 0;
+    uint32_t code_size = 0;
     uint32_t magic = 0;
-    size_t patch_num = 0;
+    uint32_t patch_num = 0;
 
     IN_UPDATE_SZ(magic);
     if (magic != magic_begin)
       return 0;
 
-    size_t name_len;
+    uint32_t name_len;
     IN_UPDATE_SZ(name_len);
     char* c_name = new char[name_len+1];
     ins.read(c_name, name_len*sizeof(char));
@@ -373,6 +451,8 @@ namespace gbe {
     name = c_name;
     delete[] c_name;
 
+    IN_UPDATE_SZ(oclVersion);
+
     IN_UPDATE_SZ(argNum);
     args = GBE_NEW_ARRAY_NO_ARG(KernelArgument, argNum);
     for (uint32_t i = 0; i < argNum; i++) {
@@ -384,7 +464,7 @@ namespace gbe {
 
       IN_UPDATE_SZ(arg.info.addrSpace);
 
-      size_t len;
+      uint32_t len;
       char* a_name = NULL;
 
       IN_UPDATE_SZ(len);
@@ -447,7 +527,7 @@ namespace gbe {
     IN_UPDATE_SZ(has_samplerset);
     if (has_samplerset) {
       samplerSet = GBE_NEW(ir::SamplerSet);
-      size_t sz = samplerSet->deserializeFromBin(ins);
+      uint32_t sz = samplerSet->deserializeFromBin(ins);
       if (sz == 0) {
         return 0;
       }
@@ -460,7 +540,7 @@ namespace gbe {
     IN_UPDATE_SZ(has_imageset);
     if (has_imageset) {
       imageSet = GBE_NEW(ir::ImageSet);
-      size_t sz = imageSet->deserializeFromBin(ins);
+      uint32_t sz = imageSet->deserializeFromBin(ins);
       if (sz == 0) {
         return 0;
       }
@@ -482,7 +562,7 @@ namespace gbe {
     if (magic != magic_end)
       return 0;
 
-    size_t total_bytes;
+    uint32_t total_bytes;
     IN_UPDATE_SZ(total_bytes);
     if (total_bytes + sizeof(total_size) != total_size)
       return 0;
@@ -570,12 +650,11 @@ namespace gbe {
     program->CleanLlvmResource();
   }
 
+  BVAR(OCL_DEBUGINFO, false);
 #ifdef GBE_COMPILER_AVAILABLE
-  BVAR(OCL_OUTPUT_BUILD_LOG, false);
-
   static bool buildModuleFromSource(const char *source, llvm::Module** out_module, llvm::LLVMContext* llvm_ctx,
-                                    std::string dumpLLVMFileName, std::vector<std::string>& options, size_t stringSize, char *err,
-                                    size_t *errSize) {
+                                    std::string dumpLLVMFileName, std::string dumpSPIRBinaryName, std::vector<std::string>& options, size_t stringSize, char *err,
+                                    size_t *errSize, uint32_t oclVersion) {
     // Arguments to pass to the clang frontend
     vector<const char *> args;
     bool bFastMath = false;
@@ -585,6 +664,17 @@ namespace gbe {
     }
 
     args.push_back("-cl-kernel-arg-info");
+    // The ParseCommandLineOptions used for mllvm args can not be used with multithread
+    // and GVN now have a 100 inst limit on block scan. Now only pass a bigger limit
+    // for each context only once, this can also fix multithread bug.
+#if LLVM_VERSION_MINOR >= 8
+    static bool ifsetllvm = false;
+    if(!ifsetllvm) {
+      args.push_back("-mllvm");
+      args.push_back("-memdep-block-scan-limit=200");
+      ifsetllvm = true;
+    }
+#endif
 #ifdef GEN7_SAMPLER_CLAMP_BORDER_WORKAROUND
     args.push_back("-DGEN7_SAMPLER_CLAMP_BORDER_WORKAROUND");
 #endif
@@ -602,10 +692,15 @@ namespace gbe {
     args.push_back("-x");
     args.push_back("cl");
     args.push_back("-triple");
-    args.push_back("spir");
+    if (oclVersion >= 200) {
+      args.push_back("spir64");
+      args.push_back("-fblocks");
+    } else
+      args.push_back("spir");
 #endif /* LLVM_VERSION_MINOR <= 2 */
     args.push_back("stringInput.cl");
-    args.push_back("-ffp-contract=off");
+    args.push_back("-ffp-contract=on");
+    if(OCL_DEBUGINFO) args.push_back("-g");
 
     // The compiler invocation needs a DiagnosticsEngine so it can report problems
     std::string ErrorString;
@@ -649,7 +744,18 @@ namespace gbe {
     clang::LangOptions & lang_opts = Clang.getLangOpts();
     lang_opts.OpenCL = 1;
     
-    GBE_ASSERT(Clang.getFrontendOpts().LLVMArgs.empty() && "We do not have llvm args now");
+    //llvm flags need command line parsing to take effect
+    if (!Clang.getFrontendOpts().LLVMArgs.empty()) {
+      unsigned NumArgs = Clang.getFrontendOpts().LLVMArgs.size();
+      const char **Args = new const char*[NumArgs + 2];
+      Args[0] = "clang (LLVM option parsing)";
+      for (unsigned i = 0; i != NumArgs; ++i){
+        Args[i + 1] = Clang.getFrontendOpts().LLVMArgs[i].c_str();
+      }
+      Args[NumArgs + 1] = 0;
+      llvm::cl::ParseCommandLineOptions(NumArgs + 1, Args);
+      delete [] Args;
+    }
   
     // Create an action and make the compiler instance carry it out
     std::unique_ptr<clang::CodeGenAction> Act(new clang::EmitLLVMOnlyAction(llvm_ctx));
@@ -695,6 +801,20 @@ namespace gbe {
         (*out_module)->print(ostream, 0);
       } //Otherwise, you'll have to make do without the dump.
     }
+
+    if (!dumpSPIRBinaryName.empty()) {
+      std::string err;
+      llvm::raw_fd_ostream ostream (dumpSPIRBinaryName.c_str(),
+                                    err,
+      #if LLVM_VERSION_MINOR == 3
+                                    0
+      #else
+                                    llvm::sys::fs::F_None
+      #endif
+                                    );
+      if (err.empty())
+        llvm::WriteBitcodeToFile(*out_module, ostream);
+    }
 #else
     if (!dumpLLVMFileName.empty()) {
       std::error_code err;
@@ -704,12 +824,21 @@ namespace gbe {
         (*out_module)->print(ostream, 0);
       } //Otherwise, you'll have to make do without the dump.
     }
+
+    if (!dumpSPIRBinaryName.empty()) {
+      std::error_code err;
+      llvm::raw_fd_ostream ostream (dumpSPIRBinaryName.c_str(),
+                                    err, llvm::sys::fs::F_None);
+      if (!err)
+        llvm::WriteBitcodeToFile(*out_module, ostream);
+    }
 #endif
     return true;
   }
 
 
   SVAR(OCL_PCH_PATH, OCL_PCH_OBJECT);
+  SVAR(OCL_PCH_20_PATH, OCL_PCH_OBJECT_20);
   SVAR(OCL_HEADER_FILE_DIR, OCL_HEADER_DIR);
   BVAR(OCL_OUTPUT_KERNEL_SOURCE, false);
 
@@ -719,19 +848,25 @@ namespace gbe {
                                      std::vector<std::string>& clOpt,
                                      std::string& dumpLLVMFileName,
                                      std::string& dumpASMFileName,
+                                     std::string& dumpSPIRBinaryName,
                                      int& optLevel,
                                      size_t stringSize,
                                      char *err,
-                                     size_t *errSize)
+                                     size_t *errSize,
+                                     uint32_t &oclVersion)
   {
-    std::string dirs = OCL_PCH_PATH;
-    std::istringstream idirs(dirs);
     std::string pchFileName;
     bool findPCH = false;
+#if defined(__ANDROID__)
+    bool invalidPCH = true;
+#else
     bool invalidPCH = false;
+#endif
     size_t start = 0, end = 0;
 
     std::string hdirs = OCL_HEADER_FILE_DIR;
+    if(hdirs == "")
+      hdirs = OCL_HEADER_DIR;
     std::istringstream hidirs(hdirs);
     std::string headerFilePath;
     bool findOcl = false;
@@ -743,6 +878,7 @@ namespace gbe {
         break;
       }
     }
+    (void) findOcl;
     assert(findOcl);
     if (OCL_OUTPUT_KERNEL_SOURCE) {
       if(options) {
@@ -750,7 +886,7 @@ namespace gbe {
         std::cout << options << std::endl;
       }
       std::cout << "CL kernel source:" << std::endl;
-      std::cout << source;
+      std::cout << source << std::endl;
     }
     std::string includePath  = "-I" + headerFilePath;
     clOpt.push_back(includePath);
@@ -758,12 +894,14 @@ namespace gbe {
 
     if (options) {
       char *c_str = (char *)malloc(sizeof(char) * (strlen(options) + 1));
+      if (c_str == NULL)
+        return false;
       memcpy(c_str, options, strlen(options) + 1);
       std::string optionStr(c_str);
       const std::string unsupportedOptions("-cl-denorms-are-zero, -cl-strict-aliasing, -cl-opt-disable,"
                        "-cl-no-signed-zeros, -cl-fp32-correctly-rounded-divide-sqrt");
 
-      const std::string uncompatiblePCHOptions = ("-cl-single-precision-constant, -cl-fast-relaxed-math, -cl-std=CL1.1, -cl-finite-math-only");
+      const std::string uncompatiblePCHOptions = ("-cl-single-precision-constant, -cl-fast-relaxed-math, -cl-std=CL1.1, -cl-finite-math-only, -cl-unsafe-math-optimizations");
       const std::string fastMathOption = ("-cl-fast-relaxed-math");
       while (end != std::string::npos) {
         end = optionStr.find(' ', start);
@@ -839,11 +977,16 @@ EXTEND_QUOTE:
 
         if(str.find("-cl-std=") != std::string::npos) {
           useDefaultCLCVersion = false;
-          if (str == "-cl-std=CL1.1")
+          if (str == "-cl-std=CL1.1") {
             clOpt.push_back("-D__OPENCL_C_VERSION__=110");
-          else if (str == "-cl-std=CL1.2")
+            oclVersion = 110;
+          } else if (str == "-cl-std=CL1.2") {
             clOpt.push_back("-D__OPENCL_C_VERSION__=120");
-          else {
+            oclVersion = 120;
+          } else if (str == "-cl-std=CL2.0") {
+            clOpt.push_back("-D__OPENCL_C_VERSION__=200");
+            oclVersion = 200;
+          } else {
             if (err && stringSize > 0 && errSize)
               *errSize = snprintf(err, stringSize, "Invalid build option: %s\n", str.c_str());
             return false;
@@ -868,14 +1011,26 @@ EXTEND_QUOTE:
           continue; // Don't push this str back; ignore it.
         }
 
+        if(str.find("-dump-spir-binary=") != std::string::npos) {
+          dumpSPIRBinaryName = str.substr(str.find("=") + 1);
+          continue; // Don't push this str back; ignore it.
+        }
+
         clOpt.push_back(str);
       }
       free(c_str);
     }
 
     if (useDefaultCLCVersion) {
+#ifdef ENABLE_OPENCL_20
+      clOpt.push_back("-D__OPENCL_C_VERSION__=200");
+      clOpt.push_back("-cl-std=CL2.0");
+      oclVersion = 200;
+#else
       clOpt.push_back("-D__OPENCL_C_VERSION__=120");
       clOpt.push_back("-cl-std=CL1.2");
+      oclVersion = 120;
+#endif
     }
     //for clCompilerProgram usage.
     if(temp_header_path){
@@ -883,6 +1038,14 @@ EXTEND_QUOTE:
       clOpt.push_back(temp_header_path);
     }
 
+    std::string dirs = OCL_PCH_PATH;
+    if(oclVersion >= 200)
+      dirs = OCL_PCH_20_PATH;
+    if(dirs == "") {
+      dirs = oclVersion >= 200 ? OCL_PCH_OBJECT_20 : OCL_PCH_OBJECT;
+    }
+    std::istringstream idirs(dirs);
+
     while (getline(idirs, pchFileName, ':')) {
       if(access(pchFileName.c_str(), R_OK) == 0) {
         findPCH = true;
@@ -912,10 +1075,12 @@ EXTEND_QUOTE:
     int optLevel = 1;
     std::vector<std::string> clOpt;
     std::string dumpLLVMFileName, dumpASMFileName;
+    std::string dumpSPIRBinaryName;
+    uint32_t oclVersion = 0;
     if (!processSourceAndOption(source, options, NULL, clOpt,
-                                dumpLLVMFileName, dumpASMFileName,
+                                dumpLLVMFileName, dumpASMFileName, dumpSPIRBinaryName,
                                 optLevel,
-                                stringSize, err, errSize))
+                                stringSize, err, errSize, oclVersion))
       return NULL;
 
     gbe_program p;
@@ -926,14 +1091,14 @@ EXTEND_QUOTE:
     if (!llvm::llvm_is_multithreaded())
       llvm_mutex.lock();
 
-    if (buildModuleFromSource(source, &out_module, llvm_ctx, dumpLLVMFileName, clOpt,
-                              stringSize, err, errSize)) {
+    if (buildModuleFromSource(source, &out_module, llvm_ctx, dumpLLVMFileName, dumpSPIRBinaryName, clOpt,
+                              stringSize, err, errSize, oclVersion)) {
     // Now build the program from llvm
       size_t clangErrSize = 0;
-      if (err != NULL) {
+      if (err != NULL && *errSize != 0) {
         GBE_ASSERT(errSize != NULL);
-        stringSize -= *errSize;
-        err += *errSize;
+        stringSize = stringSize - *errSize;
+        err = err + *errSize;
         clangErrSize = *errSize;
       }
 
@@ -942,9 +1107,10 @@ EXTEND_QUOTE:
         if (asmDumpStream)
           fclose(asmDumpStream);
       }
+
       p = gbe_program_new_from_llvm(deviceID, NULL, out_module, llvm_ctx,
                                     dumpASMFileName.empty() ? NULL : dumpASMFileName.c_str(),
-                                    stringSize, err, errSize, optLevel);
+                                    stringSize, err, errSize, optLevel, options);
       if (err != NULL)
         *errSize += clangErrSize;
       if (OCL_OUTPUT_BUILD_LOG && options)
@@ -972,9 +1138,11 @@ EXTEND_QUOTE:
     int optLevel = 1;
     std::vector<std::string> clOpt;
     std::string dumpLLVMFileName, dumpASMFileName;
+    std::string dumpSPIRBinaryName;
+    uint32_t oclVersion = 0;
     if (!processSourceAndOption(source, options, temp_header_path, clOpt,
-                                dumpLLVMFileName, dumpASMFileName,
-                                optLevel, stringSize, err, errSize))
+                                dumpLLVMFileName, dumpASMFileName, dumpSPIRBinaryName,
+                                optLevel, stringSize, err, errSize, oclVersion))
       return NULL;
 
     gbe_program p;
@@ -982,10 +1150,14 @@ EXTEND_QUOTE:
     //FIXME: if use new allocated context to link two modules there would be context mismatch
     //for some functions, so we use global context now, need switch to new context later.
     llvm::Module * out_module;
+#if LLVM_VERSION_MAJOR == 3 && LLVM_VERSION_MINOR >= 9
+    llvm::LLVMContext* llvm_ctx = &GBEGetLLVMContext();
+#else
     llvm::LLVMContext* llvm_ctx = &llvm::getGlobalContext();
+#endif
 
-    if (buildModuleFromSource(source, &out_module, llvm_ctx, dumpLLVMFileName, clOpt,
-                              stringSize, err, errSize)) {
+    if (buildModuleFromSource(source, &out_module, llvm_ctx, dumpLLVMFileName, dumpSPIRBinaryName, clOpt,
+                              stringSize, err, errSize, oclVersion)) {
     // Now build the program from llvm
       if (err != NULL) {
         GBE_ASSERT(errSize != NULL);
@@ -993,7 +1165,7 @@ EXTEND_QUOTE:
         err += *errSize;
       }
 
-      p = gbe_program_new_gen_program(deviceID, out_module, NULL);
+      p = gbe_program_new_gen_program(deviceID, out_module, NULL, NULL);
 
       if (OCL_OUTPUT_BUILD_LOG && options)
         llvm::errs() << options;
@@ -1039,6 +1211,10 @@ EXTEND_QUOTE:
       if(pos != std::string::npos) {
         s.erase(pos, strlen("-enable-link-options"));
       }
+      pos = s.find("-dump-opt-asm");
+      if(pos != std::string::npos) {
+        s.erase(pos, strlen("-dump-opt-asm"));
+      }
       args.push_back(s.c_str());
 
       // The compiler invocation needs a DiagnosticsEngine so it can report problems
@@ -1075,12 +1251,30 @@ EXTEND_QUOTE:
     program->getGlobalConstantData(mem);
   }
 
+  static size_t programGetGlobalRelocCount(gbe_program gbeProgram) {
+    if (gbeProgram == NULL) return 0;
+    const gbe::Program *program = (const gbe::Program*) gbeProgram;
+    return program->getGlobalRelocCount();
+  }
+
+  static void programGetGlobalRelocTable(gbe_program gbeProgram, char *mem) {
+    if (gbeProgram == NULL) return;
+    const gbe::Program *program = (const gbe::Program*) gbeProgram;
+    program->getGlobalRelocTable(mem);
+  }
+
   static uint32_t programGetKernelNum(gbe_program gbeProgram) {
     if (gbeProgram == NULL) return 0;
     const gbe::Program *program = (const gbe::Program*) gbeProgram;
     return program->getKernelNum();
   }
 
+  const static char* programGetDeviceEnqueueKernelName(gbe_program gbeProgram, uint32_t index) {
+    if (gbeProgram == NULL) return 0;
+    const gbe::Program *program = (const gbe::Program*) gbeProgram;
+    return program->getDeviceEnqueueKernelName(index);
+  }
+
   static gbe_kernel programGetKernelByName(gbe_program gbeProgram, const char *name) {
     if (gbeProgram == NULL) return NULL;
     const gbe::Program *program = (gbe::Program*) gbeProgram;
@@ -1139,6 +1333,8 @@ EXTEND_QUOTE:
         return (void *)(info->typeQual.c_str());
       case GBE_GET_ARG_INFO_NAME:
         return (void *)(info->argName.c_str());
+      case GBE_GET_ARG_INFO_TYPESIZE:
+        return (void *)((size_t)info->typeSize);
       default:
         assert(0);
     }
@@ -1223,12 +1419,33 @@ EXTEND_QUOTE:
     kernel->getSamplerData(samplers);
   }
 
+  static void* kernelDupProfiling(gbe_kernel gbeKernel) {
+    if (gbeKernel == NULL) return NULL;
+    const gbe::Kernel *kernel = (const gbe::Kernel*) gbeKernel;
+    return kernel->dupProfilingInfo();
+  }
+  static uint32_t kernelGetProfilingBTI(gbe_kernel gbeKernel) {
+    if (gbeKernel == NULL) return 0;
+    const gbe::Kernel *kernel = (const gbe::Kernel*) gbeKernel;
+    return kernel->getProfilingBTI();
+  }
+  static void kernelOutputProfiling(void *profiling_info, void* buf) {
+    if (profiling_info == NULL) return;
+    ir::ProfilingInfo *pi = (ir::ProfilingInfo *)profiling_info;
+    return pi->outputProfilingInfo(buf);
+  }
   static uint32_t kernelGetPrintfNum(void * printf_info) {
     if (printf_info == NULL) return 0;
     const ir::PrintfSet *ps = (ir::PrintfSet *)printf_info;
     return ps->getPrintfNum();
   }
 
+  static uint32_t kernelUseDeviceEnqueue(gbe_kernel gbeKernel) {
+    if (gbeKernel == NULL) return 0;
+    const gbe::Kernel *kernel = (const gbe::Kernel*) gbeKernel;
+    return kernel->getUseDeviceEnqueue();
+  }
+
   static void* kernelDupPrintfSet(gbe_kernel gbeKernel) {
     if (gbeKernel == NULL) return NULL;
     const gbe::Kernel *kernel = (const gbe::Kernel*) gbeKernel;
@@ -1241,33 +1458,17 @@ EXTEND_QUOTE:
     return ps->getBufBTI();
   }
 
-  static uint8_t kernelGetPrintfIndexBufBTI(void * printf_info) {
-    if (printf_info == NULL) return 0;
-    const ir::PrintfSet *ps = (ir::PrintfSet *)printf_info;
-    return ps->getIndexBufBTI();
-  }
-
   static void kernelReleasePrintfSet(void * printf_info) {
     if (printf_info == NULL) return;
     ir::PrintfSet *ps = (ir::PrintfSet *)printf_info;
     delete ps;
   }
 
-  static uint32_t kernelGetPrintfSizeOfSize(void * printf_info) {
-    if (printf_info == NULL) return 0;
-    const ir::PrintfSet *ps = (ir::PrintfSet *)printf_info;
-    return ps->getPrintfSizeOfSize();
-  }
-
-  static void kernelOutputPrintf(void * printf_info, void* index_addr,
-                                 void* buf_addr, size_t global_wk_sz0,
-                                 size_t global_wk_sz1, size_t global_wk_sz2,
-                                 size_t output_sz)
+  static void kernelOutputPrintf(void * printf_info, void* buf_addr)
   {
     if (printf_info == NULL) return;
     ir::PrintfSet *ps = (ir::PrintfSet *)printf_info;
-    ps->outputPrintf(index_addr, buf_addr, global_wk_sz0,
-                         global_wk_sz1, global_wk_sz2, output_sz);
+    ps->outputPrintf(buf_addr);
   }
 
   static void kernelGetCompileWorkGroupSize(gbe_kernel gbeKernel, size_t wg_size[3]) {
@@ -1288,6 +1489,12 @@ EXTEND_QUOTE:
     kernel->getImageData(images);
   }
 
+  static uint32_t kernelGetOclVersion(gbe_kernel gbeKernel) {
+    if (gbeKernel == NULL) return 0;
+    const gbe::Kernel *kernel = (const gbe::Kernel*) gbeKernel;
+    return kernel->getOclVersion();
+  }
+
   static uint32_t kernelGetRequiredWorkGroupSize(gbe_kernel kernel, uint32_t dim) {
     return 0u;
   }
@@ -1317,11 +1524,14 @@ GBE_EXPORT_SYMBOL gbe_program_link_from_llvm_cb *gbe_program_link_from_llvm = NU
 GBE_EXPORT_SYMBOL gbe_program_build_from_llvm_cb *gbe_program_build_from_llvm = NULL;
 GBE_EXPORT_SYMBOL gbe_program_get_global_constant_size_cb *gbe_program_get_global_constant_size = NULL;
 GBE_EXPORT_SYMBOL gbe_program_get_global_constant_data_cb *gbe_program_get_global_constant_data = NULL;
+GBE_EXPORT_SYMBOL gbe_program_get_global_reloc_count_cb *gbe_program_get_global_reloc_count = NULL;
+GBE_EXPORT_SYMBOL gbe_program_get_global_reloc_table_cb *gbe_program_get_global_reloc_table = NULL;
 GBE_EXPORT_SYMBOL gbe_program_clean_llvm_resource_cb *gbe_program_clean_llvm_resource = NULL;
 GBE_EXPORT_SYMBOL gbe_program_delete_cb *gbe_program_delete = NULL;
 GBE_EXPORT_SYMBOL gbe_program_get_kernel_num_cb *gbe_program_get_kernel_num = NULL;
 GBE_EXPORT_SYMBOL gbe_program_get_kernel_by_name_cb *gbe_program_get_kernel_by_name = NULL;
 GBE_EXPORT_SYMBOL gbe_program_get_kernel_cb *gbe_program_get_kernel = NULL;
+GBE_EXPORT_SYMBOL gbe_program_get_device_enqueue_kernel_name_cb *gbe_program_get_device_enqueue_kernel_name = NULL;
 GBE_EXPORT_SYMBOL gbe_kernel_get_name_cb *gbe_kernel_get_name = NULL;
 GBE_EXPORT_SYMBOL gbe_kernel_get_attributes_cb *gbe_kernel_get_attributes = NULL;
 GBE_EXPORT_SYMBOL gbe_kernel_get_code_cb *gbe_kernel_get_code = NULL;
@@ -1345,13 +1555,16 @@ GBE_EXPORT_SYMBOL gbe_kernel_get_sampler_data_cb *gbe_kernel_get_sampler_data =
 GBE_EXPORT_SYMBOL gbe_kernel_get_compile_wg_size_cb *gbe_kernel_get_compile_wg_size = NULL;
 GBE_EXPORT_SYMBOL gbe_kernel_get_image_size_cb *gbe_kernel_get_image_size = NULL;
 GBE_EXPORT_SYMBOL gbe_kernel_get_image_data_cb *gbe_kernel_get_image_data = NULL;
+GBE_EXPORT_SYMBOL gbe_kernel_get_ocl_version_cb *gbe_kernel_get_ocl_version = NULL;
+GBE_EXPORT_SYMBOL gbe_output_profiling_cb *gbe_output_profiling = NULL;
+GBE_EXPORT_SYMBOL gbe_dup_profiling_cb *gbe_dup_profiling = NULL;
+GBE_EXPORT_SYMBOL gbe_get_profiling_bti_cb *gbe_get_profiling_bti = NULL;
 GBE_EXPORT_SYMBOL gbe_get_printf_num_cb *gbe_get_printf_num = NULL;
 GBE_EXPORT_SYMBOL gbe_dup_printfset_cb *gbe_dup_printfset = NULL;
 GBE_EXPORT_SYMBOL gbe_get_printf_buf_bti_cb *gbe_get_printf_buf_bti = NULL;
-GBE_EXPORT_SYMBOL gbe_get_printf_indexbuf_bti_cb *gbe_get_printf_indexbuf_bti = NULL;
 GBE_EXPORT_SYMBOL gbe_release_printf_info_cb *gbe_release_printf_info = NULL;
-GBE_EXPORT_SYMBOL gbe_get_printf_sizeof_size_cb *gbe_get_printf_sizeof_size = NULL;
 GBE_EXPORT_SYMBOL gbe_output_printf_cb *gbe_output_printf = NULL;
+GBE_EXPORT_SYMBOL gbe_kernel_use_device_enqueue_cb *gbe_kernel_use_device_enqueue = NULL;
 
 #ifdef GBE_COMPILER_AVAILABLE
 namespace gbe
@@ -1366,9 +1579,12 @@ namespace gbe
       gbe_program_check_opt = gbe::programCheckOption;
       gbe_program_get_global_constant_size = gbe::programGetGlobalConstantSize;
       gbe_program_get_global_constant_data = gbe::programGetGlobalConstantData;
+      gbe_program_get_global_reloc_count = gbe::programGetGlobalRelocCount;
+      gbe_program_get_global_reloc_table = gbe::programGetGlobalRelocTable;
       gbe_program_clean_llvm_resource = gbe::programCleanLlvmResource;
       gbe_program_delete = gbe::programDelete;
       gbe_program_get_kernel_num = gbe::programGetKernelNum;
+      gbe_program_get_device_enqueue_kernel_name = gbe::programGetDeviceEnqueueKernelName;
       gbe_program_get_kernel_by_name = gbe::programGetKernelByName;
       gbe_program_get_kernel = gbe::programGetKernel;
       gbe_kernel_get_name = gbe::kernelGetName;
@@ -1394,13 +1610,16 @@ namespace gbe
       gbe_kernel_get_compile_wg_size = gbe::kernelGetCompileWorkGroupSize;
       gbe_kernel_get_image_size = gbe::kernelGetImageSize;
       gbe_kernel_get_image_data = gbe::kernelGetImageData;
+      gbe_kernel_get_ocl_version = gbe::kernelGetOclVersion;
+      gbe_get_profiling_bti = gbe::kernelGetProfilingBTI;
       gbe_get_printf_num = gbe::kernelGetPrintfNum;
+      gbe_dup_profiling = gbe::kernelDupProfiling;
+      gbe_output_profiling = gbe::kernelOutputProfiling;
       gbe_get_printf_buf_bti = gbe::kernelGetPrintfBufBTI;
-      gbe_get_printf_indexbuf_bti = gbe::kernelGetPrintfIndexBufBTI;
       gbe_dup_printfset = gbe::kernelDupPrintfSet;
-      gbe_get_printf_sizeof_size = gbe::kernelGetPrintfSizeOfSize;
       gbe_release_printf_info = gbe::kernelReleasePrintfSet;
       gbe_output_printf = gbe::kernelOutputPrintf;
+      gbe_kernel_use_device_enqueue = gbe::kernelUseDeviceEnqueue;
       genSetupCallBacks();
     }
 
