@@ -52,6 +52,13 @@ namespace ir {
     const LabelIndex index = this->label();
     this->LABEL(index);
     const BasicBlock *lastBlock = this->bb;
+
+    /* Append the STORE_PROFILING just before return. */
+    if (unit.getInProfilingMode() == true) {
+      this->STORE_PROFILING(this->getUnit().getProfilingInfo()->getBTI(),
+                            this->getUnit().getProfilingInfo()->getProfilingType());
+    }
+
     this->RET();
 
     // Now traverse all instructions and replace all returns by GOTO index
@@ -163,6 +170,8 @@ namespace ir {
 
     if (opcode == OP_LOAD) {
       LoadInstruction *load = cast<LoadInstruction>(insn);
+      if(!load)
+        return false;
       if (load->getAddressSpace() != MEM_PRIVATE)
         return false;
       loadAddImm.load = insn;
@@ -243,6 +252,7 @@ namespace ir {
     set<PushLocation> inserted;
     for (const auto &loadAddImm : seq) {
       LoadInstruction *load = cast<LoadInstruction>(loadAddImm.load);
+      if(!load) continue;
       const uint32_t valueNum = load->getValueNum();
       bool replaced = false;
       Instruction *ins_after = load; // the instruction to insert after.
@@ -303,6 +313,7 @@ namespace ir {
         Instruction *insn = const_cast<Instruction*>(use->getInstruction());
         const Opcode opcode = insn->getOpcode();
         const uint32_t dstNum = insn->getDstNum();
+        (void) dstNum;
         GBE_ASSERT(dstNum == 1 || opcode == OP_LOAD);
         const Register dst = insn->getDst();
         auto it = addPtrInsns.find(derivedRegs[i]);
@@ -316,11 +327,13 @@ namespace ir {
           derivedRegs.push_back(dst);
         } else if(opcode == OP_LOAD) {
           LoadInstruction *load = cast<LoadInstruction>(insn);
+          if(!load)
+            continue;
           if (load->getAddressSpace() != MEM_PRIVATE)
             continue;
 
           IndirectLoad indirectLoad;
-          Register addr = load->getAddress();
+          Register addr = load->getAddressRegister();
           indirectLoad.argID = argID;
           indirectLoad.load = insn;
 
@@ -357,7 +370,7 @@ namespace ir {
       const Register arg = fn->getArg(indirectLoad.argID).reg;
       if(dead.contains(indirectLoad.load)) continue;  //repetitive load in the indirectSeq, skip.
       LoadInstruction *load = cast<LoadInstruction>(indirectLoad.load);
-      const uint32_t valueNum = load->getValueNum();
+      const uint32_t valueNum = load ? load->getValueNum() : 0;
       bool replaced = false;
       Instruction *ins_after = load; // the instruction to insert after.
       for (uint32_t valueID = 0; valueID < valueNum; ++valueID) {
@@ -367,8 +380,14 @@ namespace ir {
         const uint32_t offset = valueID * size;
 
         const Register reg = load->getValue(valueID);
-
-        Instruction mov = ir::INDIRECT_MOV(type, reg, arg, load->getAddress(), offset);
+        Register addressReg = load->getAddressRegister();
+        if (fn->getPointerFamily() == FAMILY_QWORD) {
+          Register tmp = fn->newRegister(FAMILY_DWORD);
+          Instruction cvt = ir::CVT(ir::TYPE_U32, ir::TYPE_U64, tmp, load->getAddressRegister());
+          cvt.insert(ins_after, &ins_after);
+          addressReg = tmp;
+        }
+        Instruction mov = ir::INDIRECT_MOV(type, reg, arg, addressReg, offset);
         mov.insert(ins_after, &ins_after);
         replaced = true;
       }
@@ -381,7 +400,7 @@ namespace ir {
       vector<Instruction *> adds = indirectLoad.adds;
       for (uint32_t i=0; i<adds.size(); i++) {
         BinaryInstruction *add = cast<BinaryInstruction>(adds[i]);
-        if (!dead.contains(add)) {
+        if (add && !dead.contains(add)) {
           Register dst = add->getDst();
           const Register src0 = add->getSrc(0);
           const Register src1 = add->getSrc(1);
@@ -444,6 +463,7 @@ namespace ir {
       // add.ptr_type dst ptr other
       if (opcode != OP_ADD) return false;
       BinaryInstruction *add = cast<BinaryInstruction>(insn);
+      if(!add) return false;
       const Type addType = add->getType();
       const RegisterFamily family = getFamily(addType);
       if (family != unit.getPointerFamily()) return false;
@@ -460,6 +480,7 @@ namespace ir {
       Instruction *otherInsn = const_cast<Instruction*>(otherDef->getInstruction());
       if (otherInsn->getOpcode() != OP_LOADI) return false;
       LoadImmInstruction *loadImm = cast<LoadImmInstruction>(otherInsn);
+      if(!loadImm) return false;
       const Immediate imm = loadImm->getImmediate();
       const uint64_t offset = getOffsetFromImm(imm);
 
