@@ -156,7 +156,7 @@ static TypePair getExpandedIntTypes(Type *Ty) {
 
 // Return true if Val is an int which should be converted.
 static bool shouldConvert(const Value *Val) {
-  Type *Ty = Val->getType();
+  Type *Ty = Val ? Val->getType() : NULL;
   if (IntegerType *ITy = dyn_cast<IntegerType>(Ty))
     return !isLegalBitSize(ITy->getBitWidth());
   return false;
@@ -324,12 +324,14 @@ static Value *buildVectorOrScalar(ConversionState &State, IRBuilder<> &IRB, Smal
     Value * vec = NULL;
     unsigned ElemNo = Elements.size();
     Type *ElemTy = Elements[0]->getType();
+    // if it is illegal integer type, these instructions will be further
+    // splited, that's why these temporary values should be erased.
     bool KeepInsert = isLegalBitSize(ElemTy->getPrimitiveSizeInBits() * ElemNo);
     for (unsigned i = 0; i < ElemNo; ++i) {
       Value *tmp = vec ? vec : UndefValue::get(VectorType::get(ElemTy, ElemNo));
       Value *idx = ConstantInt::get(IntTy, i);
       vec = IRB.CreateInsertElement(tmp, Elements[i], idx);
-      if (!KeepInsert) {
+      if (!KeepInsert && !isa<Constant>(vec)) {
         State.addEraseCandidate(cast<Instruction>(vec));
       }
     }
@@ -388,7 +390,7 @@ static void convertInstruction(Instruction *Inst, ConversionState &State,
   // Set the insert point *after* Inst, so that any instructions inserted here
   // will be visited again. That allows iterative expansion of types > i128.
   BasicBlock::iterator InsertPos(Inst);
-  IRBuilder<> IRB(++InsertPos);
+  IRBuilder<> IRB(&*++InsertPos);
   StringRef Name = Inst->getName();
 
   if (PHINode *Phi = dyn_cast<PHINode>(Inst)) {
@@ -398,6 +400,8 @@ static void convertInstruction(Instruction *Inst, ConversionState &State,
     PHINode *Hi = IRB.CreatePHI(OpTys.Hi, N, Twine(Name + ".hi"));
     for (unsigned I = 0; I != N; ++I) {
       Value *InVal = Phi->getIncomingValue(I);
+      if(!InVal)
+        continue;
       BasicBlock *InBB = Phi->getIncomingBlock(I);
       // If the value hasn't already been converted then this is a
       // forward-reference PHI which needs to be patched up after RPO traversal.
@@ -434,6 +438,7 @@ static void convertInstruction(Instruction *Inst, ConversionState &State,
       State.recordConverted(Trunc, NewInst);
     } else {
       TypePair Tys = getExpandedIntTypes(Trunc->getType());
+      (void) OpTys;
       assert(Tys.Lo == OpTys.Lo);
       Value *Lo = Ops.Lo;
       Value *Hi = IRB.CreateTrunc(Ops.Hi, Tys.Hi, Twine(Name, ".hi"));
