@@ -41,9 +41,12 @@ using namespace llvm;
 namespace gbe
 {
   bool isKernelFunction(const llvm::Function &F) {
+    bool bKernel = false;
+#if LLVM_VERSION_MAJOR == 3 && LLVM_VERSION_MINOR >= 9
+    bKernel = F.getMetadata("kernel_arg_name") != NULL;
+#else
     const Module *module = F.getParent();
     const Module::NamedMDListType& globalMD = module->getNamedMDList();
-    bool bKernel = false;
     for(auto i = globalMD.begin(); i != globalMD.end(); i++) {
       const NamedMDNode &md = *i;
       if(strcmp(md.getName().data(), "opencl.kernels") != 0) continue;
@@ -58,9 +61,31 @@ namespace gbe
         if(op == &F) bKernel = true;
       }
     }
+#endif
     return bKernel;
   }
 
+  uint32_t getModuleOclVersion(const llvm::Module *M) {
+    uint32_t oclVersion = 120;
+    NamedMDNode *version = M->getNamedMetadata("opencl.ocl.version");
+    if (version == NULL)
+      return oclVersion;
+    uint32_t ops = version->getNumOperands();
+    if(ops > 0) {
+      uint32_t major = 0, minor = 0;
+      MDNode* node = version->getOperand(0);
+#if LLVM_VERSION_MAJOR == 3 && LLVM_VERSION_MINOR >= 6
+      major = mdconst::extract<ConstantInt>(node->getOperand(0))->getZExtValue();
+      minor = mdconst::extract<ConstantInt>(node->getOperand(1))->getZExtValue();
+#else
+      major = cast<ConstantInt>(node->getOperand(0))->getZExtValue();
+      minor = cast<ConstantInt>(node->getOperand(1))->getZExtValue();
+#endif
+      oclVersion = major * 100 + minor * 10;
+    }
+    return oclVersion;
+  }
+
   int32_t getPadding(int32_t offset, int32_t align) {
     return (align - (offset % align)) % align; 
   }
@@ -222,7 +247,9 @@ namespace gbe
   {
     const uint32_t ptrSize = unit.getPointerSize();
     Value* parentPointer = GEPInst->getOperand(0);
-    CompositeType* CompTy = cast<CompositeType>(parentPointer->getType());
+    CompositeType* CompTy = parentPointer ? cast<CompositeType>(parentPointer->getType()) : NULL;
+    if(!CompTy)
+      return false;
 
     Value* currentAddrInst = 
       new PtrToIntInst(parentPointer, IntegerType::get(GEPInst->getContext(), ptrSize), "", GEPInst);
@@ -254,6 +281,9 @@ namespace gbe
 
         Value *operand = GEPInst->getOperand(op); 
 
+        if(!operand)
+          continue;
+#if 0
         //HACK TODO: Inserted by type replacement.. this code could break something????
         if(getTypeByteSize(unit, operand->getType())>4)
         {
@@ -277,7 +307,7 @@ namespace gbe
                   "", GEPInst);
           }
         }
-
+#endif
         Value* tmpMul = operand;
         if (size != 1) {
           tmpMul = BinaryOperator::Create(Instruction::Mul, newConstSize, operand,
