@@ -47,8 +47,13 @@ namespace gbe {
         AU.addPreservedID(LoopSimplifyID);
         AU.addRequiredID(LCSSAID);
         AU.addPreservedID(LCSSAID);
+#if LLVM_VERSION_MAJOR == 3 &&  LLVM_VERSION_MINOR >= 8
+        AU.addRequired<ScalarEvolutionWrapperPass>();
+        AU.addPreserved<ScalarEvolutionWrapperPass>();
+#else
         AU.addRequired<ScalarEvolution>();
         AU.addPreserved<ScalarEvolution>();
+#endif
       // FIXME: Loop unroll requires LCSSA. And LCSSA requires dom info.
       // If loop unroll does not preserve dom info then LCSSA pass on next
       // loop will receive invalid dom info.
@@ -60,8 +65,8 @@ namespace gbe {
       // Returns the value associated with the given metadata node name (for
       // example, "llvm.loop.unroll.count").  If no such named metadata node
       // exists, then nullptr is returned.
-      static const ConstantInt *GetUnrollMetadataValue(const Loop *L,
-                                                     StringRef Name) {
+      static const MDNode *GetUnrollMetadataValue(const Loop *L,
+                                                          StringRef Name) {
         MDNode *LoopID = L->getLoopID();
         if (!LoopID) return nullptr;
         // First operand should refer to the loop id itself.
@@ -73,26 +78,36 @@ namespace gbe {
           const MDString *S = dyn_cast<MDString>(MD->getOperand(0));
           if (!S) continue;
           if (Name.equals(S->getString())) {
-            assert(MD->getNumOperands() == 2 &&
-                   "Unroll hint metadata should have two operands.");
-#if LLVM_VERSION_MAJOR == 3 && LLVM_VERSION_MINOR >= 6
-            return mdconst::extract<ConstantInt>(MD->getOperand(1));
-#else
-            return cast<ConstantInt>(MD->getOperand(1));
-#endif
+            return MD;
           }
         }
         return nullptr;
       }
 
+      static unsigned GetUnrollCount(const Loop *L,
+                                            StringRef Name) {
+        const MDNode *MD = GetUnrollMetadataValue(L, "llvm.loop.unroll.count");
+        if (MD) {
+          assert(MD->getNumOperands() == 2 &&
+                 "Unroll count hint metadata should have two operands.");
+          unsigned Count;
+#if LLVM_VERSION_MAJOR == 3 && LLVM_VERSION_MINOR >= 6
+          Count = mdconst::extract<ConstantInt>(MD->getOperand(1))->getZExtValue();
+#else
+          Count = cast<ConstantInt>(MD->getOperand(1))->getZExtValue();
+#endif
+          assert(Count >= 1 && "Unroll count must be positive.");
+          return Count;
+        }
+        return 0;
+      }
+
       void setUnrollID(Loop *L, bool enable) {
-        if (!enable && disabledLoops.find(L) != disabledLoops.end())
-           return;
+        assert(enable);
         LLVMContext &Context = L->getHeader()->getContext();
 #if LLVM_VERSION_MAJOR == 3 && LLVM_VERSION_MINOR >= 6
         SmallVector<Metadata *, 2> forceUnroll;
         forceUnroll.push_back(MDString::get(Context, "llvm.loop.unroll.enable"));
-        forceUnroll.push_back(ConstantAsMetadata::get(ConstantInt::get(Type::getInt1Ty(Context), enable)));
         MDNode *forceUnrollNode = MDNode::get(Context, forceUnroll);
         SmallVector<Metadata *, 4> Vals;
         Vals.push_back(NULL);
@@ -110,8 +125,6 @@ namespace gbe {
         // Set operand 0 to refer to the loop id itself.
         NewLoopID->replaceOperandWith(0, NewLoopID);
         L->setLoopID(NewLoopID);
-        if (!enable)
-          disabledLoops.insert(L);
       }
 
       static bool hasPrivateLoadStore(Loop *L) {
@@ -156,7 +169,12 @@ namespace gbe {
       // be unrolled.
       bool handleParentLoops(Loop *L, LPPassManager &LPM) {
         Loop *currL = L;
+#if LLVM_VERSION_MAJOR == 3 &&  LLVM_VERSION_MINOR >= 8
+        ScalarEvolution *SE = &getAnalysis<ScalarEvolutionWrapperPass>().getSE();
+        LoopInfo &loopInfo = getAnalysis<LoopInfoWrapperPass>().getLoopInfo();
+#else
         ScalarEvolution *SE = &getAnalysis<ScalarEvolution>();
+#endif
         BasicBlock *ExitBlock = currL->getLoopLatch();
         if (!ExitBlock || !L->isLoopExiting(ExitBlock))
           ExitBlock = currL->getExitingBlock();
@@ -166,6 +184,13 @@ namespace gbe {
         if (ExitBlock)
           currTripCount = SE->getSmallConstantTripCount(L, ExitBlock);
 
+        if (currTripCount > 32) {
+          shouldUnroll = false;
+          //Don't change the unrollID if doesn't force unroll.
+          //setUnrollID(currL, false);
+          return shouldUnroll;
+        }
+
         while(currL) {
           Loop *parentL = currL->getParentLoop();
           unsigned parentTripCount = 0;
@@ -177,16 +202,18 @@ namespace gbe {
             if (parentExitBlock)
               parentTripCount = SE->getSmallConstantTripCount(parentL, parentExitBlock);
           }
-          if ((parentTripCount != 0 && currTripCount / parentTripCount > 16) ||
-              (currTripCount > 32)) {
-            if (currL == L)
-              shouldUnroll = false;
-            setUnrollID(currL, false);
-            if (currL != L)
-              LPM.deleteLoopFromQueue(currL);
+          if (parentTripCount != 0 && currTripCount * parentTripCount > 32) {
+            //Don't change the unrollID if doesn't force unroll.
+            //setUnrollID(parentL, false);
+#if LLVM_VERSION_MAJOR == 3 &&  LLVM_VERSION_MINOR >= 8
+            loopInfo.markAsRemoved(parentL);
+#else
+            LPM.deleteLoopFromQueue(parentL);
+#endif
+            return shouldUnroll;
           }
           currL = parentL;
-          currTripCount = parentTripCount;
+          currTripCount = parentTripCount * currTripCount;
         }
         return shouldUnroll;
       }
@@ -195,11 +222,11 @@ namespace gbe {
       // some private load or store, we change it's loop meta data
       // to indicate more aggresive unrolling on it.
       virtual bool runOnLoop(Loop *L, LPPassManager &LPM) {
-        const ConstantInt *Enable = GetUnrollMetadataValue(L, "llvm.loop.unroll.enable");
+        const MDNode *Enable = GetUnrollMetadataValue(L, "llvm.loop.unroll.enable");
         if (Enable)
           return false;
-        const ConstantInt *Count = GetUnrollMetadataValue(L, "llvm.loop.unroll.count");
-        if (Count)
+        const unsigned Count = GetUnrollCount(L, "llvm.loop.unroll.count");
+        if (Count > 0)
           return false;
 
         if (!handleParentLoops(L, LPM))
@@ -214,8 +241,6 @@ namespace gbe {
       virtual const char *getPassName() const {
         return "SPIR backend: custom loop unrolling pass";
       }
-    private:
-      std::set<Loop *> disabledLoops;
 
     };
 
