@@ -253,6 +253,7 @@
 #define PCI_CHIP_SKYLAKE_ULX_GT2	0x191E   /* Intel(R) Skylake ULX - GT2 */
 #define PCI_CHIP_SKYLAKE_DT_GT1		0x1902   /* Intel(R) Skylake Desktop - GT1 */
 #define PCI_CHIP_SKYLAKE_DT_GT2		0x1912   /* Intel(R) Skylake Desktop - GT2 */
+#define PCI_CHIP_SKYLAKE_DT_GT4   0x1932   /* Intel(R) Skylake Desktop - GT4 */
 #define PCI_CHIP_SKYLAKE_HALO_GT1 	0x190B   /* Intel(R) Skylake HALO - GT1 */
 #define PCI_CHIP_SKYLAKE_HALO_GT2	0x191B   /* Intel(R) Skylake HALO - GT2 */
 #define PCI_CHIP_SKYLAKE_HALO_GT3	0x192B   /* Intel(R) Skylake HALO - GT3 */
@@ -261,6 +262,9 @@
 #define PCI_CHIP_SKYLAKE_SRV_GT2	0x191A   /* Intel(R) Skylake Server - GT2 */
 #define PCI_CHIP_SKYLAKE_SRV_GT3	0x192A   /* Intel(R) Skylake Server - GT3 */
 #define PCI_CHIP_SKYLAKE_SRV_GT4	0x193A   /* Intel(R) Skylake Server - GT4 */
+#define PCI_CHIP_SKYLAKE_WKS_GT2  0x191D   /* Intel(R) Skylake WKS - GT2 */
+#define PCI_CHIP_SKYLAKE_MEDIA_SRV_GT3  0x192D /* Intel(R) Skylake Media Server - GT3 */
+#define PCI_CHIP_SKYLAKE_WKS_GT4  0x193D   /* Intel(R) Skylake WKS - GT4 */
 
 #define IS_SKL_GT1(devid)               \
   (devid == PCI_CHIP_SKYLAKE_ULT_GT1 ||   \
@@ -275,26 +279,89 @@
    devid == PCI_CHIP_SKYLAKE_ULX_GT2 || \
    devid == PCI_CHIP_SKYLAKE_DT_GT2 || \
    devid == PCI_CHIP_SKYLAKE_HALO_GT2 || \
-   devid == PCI_CHIP_SKYLAKE_SRV_GT2)
+   devid == PCI_CHIP_SKYLAKE_SRV_GT2 || \
+   devid == PCI_CHIP_SKYLAKE_WKS_GT2)
 
 #define IS_SKL_GT3(devid)               \
   (devid == PCI_CHIP_SKYLAKE_ULT_GT3 ||   \
    devid == PCI_CHIP_SKYLAKE_HALO_GT3 || \
-   devid == PCI_CHIP_SKYLAKE_SRV_GT3)
+   devid == PCI_CHIP_SKYLAKE_SRV_GT3 || \
+   devid == PCI_CHIP_SKYLAKE_MEDIA_SRV_GT3)
 
 #define IS_SKL_GT4(devid)               \
-  (devid == PCI_CHIP_SKYLAKE_HALO_GT4 || \
-   devid == PCI_CHIP_SKYLAKE_SRV_GT4)
+  (devid == PCI_CHIP_SKYLAKE_DT_GT4 ||  \
+   devid == PCI_CHIP_SKYLAKE_HALO_GT4 || \
+   devid == PCI_CHIP_SKYLAKE_SRV_GT4 || \
+   devid == PCI_CHIP_SKYLAKE_WKS_GT4)
 
 #define IS_SKYLAKE(devid) (IS_SKL_GT1(devid) || IS_SKL_GT2(devid) || IS_SKL_GT3(devid) || IS_SKL_GT4(devid))
 
 /* BXT */
-#define PCI_CHIP_BROXTON_P	0x5A84   /* Intel(R) BXT-P for mobile desktop */
+#define PCI_CHIP_BROXTON_0	0x5A84
+#define PCI_CHIP_BROXTON_1	0x5A85
+#define PCI_CHIP_BROXTON_2	0x1A84
+#define PCI_CHIP_BROXTON_3	0x1A85
 
 #define IS_BROXTON(devid)               \
-  (devid == PCI_CHIP_BROXTON_P)
+  (devid == PCI_CHIP_BROXTON_0 ||       \
+   devid == PCI_CHIP_BROXTON_1 ||       \
+   devid == PCI_CHIP_BROXTON_2 ||       \
+   devid == PCI_CHIP_BROXTON_3)
 
-#define IS_GEN9(devid)      (IS_SKYLAKE(devid) || IS_BROXTON(devid))
+#define PCI_CHIP_KABYLAKE_ULT_GT1     0x5906
+#define PCI_CHIP_KABYLAKE_ULT_GT2     0x5916
+#define PCI_CHIP_KABYLAKE_ULT_GT3     0x5926
+#define PCI_CHIP_KABYLAKE_ULT_GT15    0x5913
+#define PCI_CHIP_KABYLAKE_ULT_GT2_1   0x5921
+#define PCI_CHIP_KABYLAKE_ULT_GT3_1   0x5923
+#define PCI_CHIP_KABYLAKE_ULT_GT3_2   0x5927
+#define PCI_CHIP_KABYLAKE_DT_GT1      0x5902
+#define PCI_CHIP_KABYLAKE_DT_GT2      0x5912
+#define PCI_CHIP_KABYLAKE_DT_GT15     0x5917
+#define PCI_CHIP_KABYLAKE_HALO_GT1    0x590B
+#define PCI_CHIP_KABYLAKE_HALO_GT2    0x591B
+#define PCI_CHIP_KABYLAKE_HALO_GT4    0x593B
+#define PCI_CHIP_KABYLAKE_HALO_GT15   0x5908
+#define PCI_CHIP_KABYLAKE_ULX_GT1     0x590E
+#define PCI_CHIP_KABYLAKE_ULX_GT2     0x591E
+#define PCI_CHIP_KABYLAKE_ULX_GT15    0x5915
+#define PCI_CHIP_KABYLAKE_SRV_GT1     0x590A
+#define PCI_CHIP_KABYLAKE_SRV_GT2     0x591A
+#define PCI_CHIP_KABYLAKE_WKS_GT2     0x591D
+
+#define IS_KBL_GT1(devid)                 \
+  (devid == PCI_CHIP_KABYLAKE_ULT_GT1 ||  \
+   devid == PCI_CHIP_KABYLAKE_DT_GT1 ||   \
+   devid == PCI_CHIP_KABYLAKE_HALO_GT1 || \
+   devid == PCI_CHIP_KABYLAKE_ULX_GT1 ||  \
+   devid == PCI_CHIP_KABYLAKE_SRV_GT1)
+
+#define IS_KBL_GT15(devid)                 \
+  (devid == PCI_CHIP_KABYLAKE_ULT_GT15 ||  \
+   devid == PCI_CHIP_KABYLAKE_DT_GT15 ||   \
+   devid == PCI_CHIP_KABYLAKE_HALO_GT15 || \
+   devid == PCI_CHIP_KABYLAKE_ULX_GT15)
+
+#define IS_KBL_GT2(devid)                  \
+  (devid == PCI_CHIP_KABYLAKE_ULT_GT2 ||   \
+   devid == PCI_CHIP_KABYLAKE_ULT_GT2_1 || \
+   devid == PCI_CHIP_KABYLAKE_DT_GT2 ||    \
+   devid == PCI_CHIP_KABYLAKE_HALO_GT2 ||  \
+   devid == PCI_CHIP_KABYLAKE_ULX_GT2 ||   \
+   devid == PCI_CHIP_KABYLAKE_SRV_GT2 ||   \
+   devid == PCI_CHIP_KABYLAKE_WKS_GT2)
+
+#define IS_KBL_GT3(devid)                  \
+  (devid == PCI_CHIP_KABYLAKE_ULT_GT3 ||   \
+   devid == PCI_CHIP_KABYLAKE_ULT_GT3_1 || \
+   devid == PCI_CHIP_KABYLAKE_ULT_GT3_2)
+
+#define IS_KBL_GT4(devid)               \
+  (devid == PCI_CHIP_KABYLAKE_HALO_GT4)
+
+#define IS_KABYLAKE(devid) (IS_KBL_GT1(devid) || IS_KBL_GT15(devid) || IS_KBL_GT2(devid) || IS_KBL_GT3(devid) || IS_KBL_GT4(devid))
+
+#define IS_GEN9(devid)     (IS_SKYLAKE(devid) || IS_BROXTON(devid) || IS_KABYLAKE(devid))
 
 #endif /* __CL_DEVICE_DATA_H__ */
 
